package jalview.analysis.scoremodels;

import jalview.api.analysis.ScoreModelI;
import jalview.api.analysis.SimilarityParamsI;
import jalview.datamodel.AlignmentView;
import jalview.math.MatrixI;

public abstract class SimilarityScoreModel implements ScoreModelI
{

  /**
   * Computed similarity scores are converted to distance scores by subtracting
   * every value from the maximum value. That is, maximum similarity corresponds
   * to zero distance, and smaller similarities to larger distances.
   */
  @Override
  public MatrixI findDistances(AlignmentView seqData,
          SimilarityParamsI options)
  {
    MatrixI similarities = findSimilarities(seqData, options);

    MatrixI distances = similarityToDistance(similarities);

    return distances;
  }

  /**
   * Converts a matrix of similarity scores to distance scores, by reversing the
   * range of the scores, mapping the maximum to zero. The input matrix is not
   * modified.
   * 
   * @param similarities
   */
  public static MatrixI similarityToDistance(MatrixI similarities)
  {
    MatrixI distances = similarities.copy();

    distances.reverseRange(true);

    return distances;
  }

}
